package com.ca.chorus.teiid.translator.async;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.teiid.language.Command;
import org.teiid.language.QueryExpression;
import org.teiid.metadata.RuntimeMetadata;
import org.teiid.translator.DataNotAvailableException;
import org.teiid.translator.Execution;
import org.teiid.translator.ExecutionContext;
import org.teiid.translator.ResultSetExecution;
import org.teiid.translator.ReusableExecution;
import org.teiid.translator.TranslatorException;

public class MessageReceiveExecution implements ResultSetExecution, ReusableExecution<Connection>  
{	
	private ExecutionContext context;
	private DataNotAvailableException callNext;
	private List<Object> msgRow = null;
	
	public MessageReceiveExecution(QueryExpression expr,
			ExecutionContext executionContext, RuntimeMetadata metadata,
			Object connection) 
	{
		this.context = executionContext;
		setWakeUpCall();
	}
	
	public void execute() throws TranslatorException {
		System.out.println("!!!! Execute");
		this.callNext = DataNotAvailableException.NO_POLLING;
		this.callNext.setStrict(true);
		setMessage("MSG_EXEC");
	}

	
	public List<?> next() throws TranslatorException, DataNotAvailableException {
		System.out.println("!!!! Next");
		System.out.println("Exception: " + callNext);
		if (callNext != null) {
			DataNotAvailableException e = callNext;
			callNext = null;
			throw e;
		}
		System.out.println("msgRow: " + msgRow);
		if (msgRow != null) {
			List<Object> results = msgRow;
			msgRow = null;
			return results;
		}
		return null;
	}

	public void reset(Command c, ExecutionContext executionContext,
			Connection connection) {
			System.out.println("!!! Reset");		
	}

	public void dispose() {
		System.out.println("!!! Dispose");
	}

	public void close() {
		System.out.println("!!! Close");
	}

	public void cancel() throws TranslatorException {
		System.out.println("!!! Cancel");
	}
	
	public void setWakeUpCall()
	{
		System.out.println("Set wake up call");
		WakeCaller w = new WakeCaller(this);
		w.schedule(this.context);
	}
	
	public void setMessage(String msg)
	{
		msgRow = new ArrayList<Object>();
		msgRow.add(msg);
	}
	
}

class WakeCaller {

	MessageReceiveExecution execution;
	public WakeCaller(MessageReceiveExecution messageReceiveExecution) 
	{
		this.execution = messageReceiveExecution;
	}

	public void schedule(final ExecutionContext context) 
	{
		ScheduledExecutorService svc = Executors.newScheduledThreadPool(1);
		svc.scheduleWithFixedDelay(new Runnable(){
			public void run() 
			{
				System.out.println("Send wake call");
				execution.setMessage("MSG_WAKE");
				context.dataAvailable();						
			}			
		}, 5, 5, TimeUnit.SECONDS);
	}
}
